#!/usr/bin/python

"""A command-line radio setup script for radios with SiK software, like the RFD900 or 3DR."""
# written to communicate with SiK version 2.6 (multipoint firmware)
# written for use with Reach image v1.2
# -which has python 2.7.3
# -and pyserial module version 2.4

######
## changelog
######
# v0.0.7 add differentiation between remote and local radio, and add ability to set air data rate
# v0.0.6 add function to test for correct baud rate
# v0.0.5 add command line options
# v0.0.4 add comments, check if serial port is open
# v0.0.3 interacting with serial port
# v0.0.2 back to basics.  write to port only
# v0.0.1 copy/pasted a function to write to serial port and read back characters


import serial, time, re
from optparse import OptionParser

######
## constants
######
serial_speeds = {2400: 2, 4800: 4, 9600: 9, 19200: 19, 38400: 38, 57600: 57, 115200: 115}
air_speeds = (4, 8, 16, 24, 32, 64, 96, 128, 192, 250)
txpowers = range(1,31)


######
## command line options
######
parser = OptionParser(usage="%prog [-p port_filepath] [-b baud]", version="%prog v0.0.5")
parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="Enables extra information output", default="False")
parser.add_option("-p", "--port", action="store", type="string", dest="port", help="Serial port filepath", default="/dev/ttyMFD2")
parser.add_option("-t", "--test-baud", action="store_true", dest="test_baud", help="Test serial port for correct baud rate", default=False)
parser.add_option("-l", "--local", action="store_true", dest="local_radio", help="Work with the local radio", default=True)
parser.add_option("-r", "--remote", action="store_false", dest="local_radio", help="Work with the remote radio")
parser.add_option("-b", "--baud", action="store", type="int", dest="baud", help="Speed in baud.  Valid speeds: %s.  If no baud specified, it will test." % (serial_speeds.keys()), default="0")
parser.add_option("--set-adr", action="store", type="int", dest="set_adr", help="Set the air data rate. Valid speeds: " + ', '.join(map(str, air_speeds)), default=None)

(options, args) = parser.parse_args()

######
## this function lets the program either be terse or speak freely
######
# use vprint() for verbose messages and print() for standard program output
if options.verbose is True:
    def vprint(*args):
        # Print each argument separately so caller doesn't need to
        # stuff everything to be printed into a single string
        for arg in args:
           print arg,
        print
else:   
    vprint = lambda *a: None      # a do-nothing function

######
## this function checks if the respose contains the string "OK"
######

def check_OK(response):
    """Checks for an "OK" response within a string."""
    pattern = '\[[0-9]\]\sOK'
    if re.search(pattern,response):
        return True
    else:
        return False

######
## this function gets a response from the serial port
######

def get_response():
    """Gets a response from the serial port."""
    sleep_time_after_buffer_read = 2
    inBuffer = ser.inWaiting()
    vprint("Characters in receive buffer before reading:", inBuffer)
    response = ""
    while inBuffer > 0:
        vprint("Reading serial port buffer.")
        response = response + ser.readline(inBuffer)
        vprint("Response:", response)
        time.sleep(sleep_time_after_buffer_read)
        inBuffer = ser.inWaiting()
        vprint("Characters in receive buffer after reading and waiting %d seconds:" % sleep_time_after_buffer_read, ser.inWaiting())
    vprint("No more characters in serial port buffer.")
    return response

######
## this function puts the radio in command mode
######
def command_mode():
    """Enters command mode"""
    ser.flushOutput()
    ser.flushInput()
    time.sleep(1)           # give the flush a second
    command = "\r\n"        # the ATO command must start on a newline
    ser.write(command)
    vprint("Sent command: (newline and carriage return)", command)
    time.sleep(0.5)
    command = "ATO\r\n"     # exit AT command mode if we are in it
    ser.write(command)      
    vprint("Sent command:", command)
    time.sleep(1)
    command = "ATI\r\n"     # test to see if we are stuck in AT command mode.  If so, we see a response from this.
    vprint("Sent command:", command)
    time.sleep(2)           # minimum 1 second wait needed before +++
    command = "+++"         # +++ enters AT command mode
    ser.write(command)
    vprint("Sent command:", command)
    time.sleep(5)           # minimum 1 second wait after +++
    response = get_response()
    if check_OK(response):
        return True
    else:
        return False
 
# Notes about serial port modes:
# when opening the serial port,
# possible timeout values:
#    1. None: wait forever, block call
#    2. 0: non-blocking mode, return immediately
#    3. x, x is bigger than 0, float allowed, timeout block call

######
## this function tries to connect at each possible baud rate until it gets a sucessful response
######
def test_baud():
    for test_baud in reversed(serial_speeds.keys()):
        ser.baudrate = test_baud
        ser.open()
        vprint("Testing serial port at", test_baud, "baud.")
        if command_mode():
            vprint("Test passed at", test_baud, "baud.")
            return(test_baud)   # return the baud value and leave serial port open
        vprint("Test failed at", test_baud, "baud.")
        ser.close()
    vprint("Could not determine baud rate!  Exiting.")
    exit(100)           

######
## the main program
######
ser = serial.Serial()   # ser is now a global variable
ser.port = options.port
ser.timeout = 0
vprint("Serial port is:", ser.portstr)

# set for local or remote radio
if options.local_radio is True:
    command_prefix = "AT"
else:
    command_prefix = "RT"

# are we testing for baudrate only?
if options.test_baud is True:
    baud = test_baud()
    ser.close()
    print(baud)
    exit(0)          # if test_baud was specified, then exit after test

# was baudrate explicitly set?  if not, test for baudrate
if options.baud == 0:
    baud = test_baud()
else:
    if options.baud in serial_speeds:
        baud = options.baud
    else:
        vprint(options.baud, " baud is not a valid speed.")
        exit(101)
ser.baudrate = baud
vprint("Serial port speed set to", baud, "baud.")

# print the serial port settings
vprint("Serial port settings:\n\t", ser)

# open the serial port
ser.open()
vprint("Serial port", ser.portstr, "opened.")

# enter command mode
command_mode()

# flush the input and output buffers
ser.flushOutput()
ser.flushInput()
time.sleep(1)           # give the flush a second. //Reason for this?

######
## DO ALL THESE THINGS
######
#  1. set SERIAL_SPEED
#  2. set AIR_SPEED
#  3. set NET_ID
#  4. set TXPOWER
#  5. set ECC
#  6. set MAVLINK
#  7. set OPPRESEND
#  8. set MIN_FREQ
#  9. set MAX_FREQ
# 10. set NUM_CHANNELS
# 11. set DUTY_CYCLE
# 12. set LBT_RSSI
# 13. set MANCHESTER
# 14. set RTSCTS
# 15. set NODEID
# 16. set NODEDESTINATION
# 17. set SYNCANY
# 18. set NODECOUNT

#  1. set SERIAL_SPEED

######
##  2. set AIR_SPEED
######

if options.set_adr != None:
    if options.set_adr in air_speeds:
        vprint("Setting AIR DATA RATE to %dkbit/s." % options.set_adr)
        command = "%sS2=%d\r\n" % (command_prefix, options.set_adr)
        vprint("Sending command: ", command)
        ser.write(command)
        time.sleep(2)
        response = get_response()
        if  not check_OK(response):
            exit(103)
    else:
        vprint("Invalid air data rate.")
        exit(102)

#  3. set NET_ID
#  4. set TXPOWER
#  5. set ECC
#  6. set MAVLINK
#  7. set OPPRESEND
#  8. set MIN_FREQ
#  9. set MAX_FREQ
# 10. set NUM_CHANNELS
# 11. set DUTY_CYCLE
# 12. set LBT_RSSI
# 13. set MANCHESTER
# 14. set RTSCTS
# 15. set NODEID
# 16. set NODEDESTINATION
# 17. set SYNCANY
# 18. set NODECOUNT

# write to EEPROM and reboot
command = "%s&W\r\n" % command_prefix
vprint("Sending command: ", command)
ser.write(command)
time.sleep(2)
response = get_response()
check_OK(response)
command = "%sZ\r\n" % command_prefix
vprint("Sending command: ", command)
ser.write(command)

# close the serial port
ser.close()
vprint("Serial port", ser.portstr, "closed.")

